from osc import cmdln

@cmdln.option('--summary-only', action='store_true',
              help="only print a summary, avoiding detailed information about specific packages")

@cmdln.option('--package', metavar='PACKAGE',
              help='only patches for this package')

def do_listpatches(self, subcmd, opts, project):
    """${cmd_name}: Display a summary of patches within a package or repository
    
    This command lists the patches on the packages included on the given
    repository

    Examples:
       listpatches GNOME:UNSTABLE                    # list patches for all projects
       listpatches --package=abiword GNOME:UNSTABLE  # list patches for a single package
       listpatches --summary-only GNOME:UNSTABLE     # display only the summary of patches found per package
    
    ${cmd_usage}
    ${cmd_option_list}
    """

    #only_this_package = opts.package
    #if (opts.package != ""):
    #    bug_summary (opts.package)
    #    return

    patch_count = 0
    package_count = 0
    packages = meta_get_packagelist(conf.config['apiurl'], project)

    summary_only = opts.summary_only

    summary = []
    for package in packages:
        if opts.package and package != opts.package:
            continue
        cnt = 0
        if not summary_only:
            print "\nPatches for %s" % package

        files = meta_get_filelist(conf.config['apiurl'], project, package)
        for file in files:
            extension = os.path.splitext(file)[1]
            if extension == ".patch" or extension == ".diff" or extension == ".dif":
                patch_count += 1
                cnt += 1
                if not summary_only:
                    print "\t%s" % file
        if not summary_only:
            print "    Number of patches found: %d" % cnt

        package_count += 1
        summary.append((package, cnt))

    if package_count == 0:
        print "Package not found."

    if package_count > 1:
        print "\n\nTotal number of packages found: %d" % package_count
        print "Total number of patches found: %d" % patch_count
        print "Packages:"
        summary.sort(cmp=lambda x, y: cmp(y[1], x[1]))
        for item in summary:
            print "\t%s: %d" % (item[0], item[1])
    else:
        print "\n\nNo packages found"
