from osc import cmdln

# cmdln.py has the limitation that *args can't start with '--'.  So we take
# some common extra args into account here, instead.  How lame.
# Fortunately, at least we can copy and paste them from osc/commandline.py.
# When updating this list, don't forget to update frob_extra_args accordingly.

@cmdln.option("-v", "--verbose", action='store_true',
              help="Display more information about what's going on")
@cmdln.option('-s', '--satisfy-from-cache-liberally', metavar='strictness', # action='store_true',
              help="Liberally satisfy build deps from cache, if possible; strictness can be 'strict', 'loose', or 'insane'")
@cmdln.option("-p", "--platform", metavar="project",
              help="Override the default project of Factory")
@cmdln.option('--noinit', '--no-init', action='store_true',
              help='Skip initialization of build root and start with build immediately.')
@cmdln.option('--no-changelog', action='store_true',
              help='don\'t update the package changelog from a changes file')
@cmdln.option('--nochecks', '--no-checks', action='store_true',
              help='Do not run post build checks on the resulting packages.')
@cmdln.option('--no-verify', action='store_true',
              help='Skip signature verification of packages used for build.')
@cmdln.option('', '--local-package', action='store_true',
              help='build a package which does not exist on the server')
@cmdln.option('-k', '--keep-pkgs', metavar='DIR',
              help='Save built packages into this directory')
#@cmdln.option('-x', '--extra-pkgs', metavar='PAC', action='append',
#              help='Add this package when installing the build-root')
@cmdln.option('-j', '--jobs', metavar='N',
              help='Compile with N jobs')
@cmdln.option('--userootforbuild', action='store_true',
              help='Run build as root. The default is to build as '
              'unprivileged user. Note that a line "# norootforbuild" '
              'in the spec file will invalidate this option.')
#@cmdln.option('', '--alternative-project', metavar='PROJECT',
#              help='specify the build target project')
@cmdln.option('-d', '--debuginfo', action='store_true',
              help='also build debuginfo sub-packages')
@cmdln.option('--clean', action='store_true',
              help='Delete old build root before initializing it')
@cmdln.option('-b', '--baselibs', action='store_true',
              help='Create -32bit/-64bit/-x86 rpms for other architectures')

def do_mawbuild(self, subcmd, opts, platform, spec):
    """${cmd_name}: A comfortable wrapper around osc build.
    
    This is mean to be a comfortable wrapper around osc build,
    with convenient defaults baked in.
    
    The use of this plugin requires you to add some settings to your ~/.oscrc.  It
    also suggests some others, but they're strictly a convenience and not absolutely
    necessary.
    
    Mandatory settings:
    
    eb-chroot-dir: This is the directory where your build chroots will be located.

    Example:

        eb-chroot-dir = /abuild

    eb-packagecache-dir: This is the directory where downloaded packages are to be cached.

    Example:
    
        eb-packagecache-dir = /cache/packagecache

    Optional settings:
    
    eb-extra-packs: Extra packs to be installed in your chroot by default.

    Example:
  
        eb-extra-packs = bzr emacs-nox gdb
     
    eb-local-packs-dir: Choose the directory whose packages will override downloaded packages.

    ${cmd_usage}
    ${cmd_option_list}
    """

    def frob_extra_args(opts):
        
        extra_args = ""
        if opts.clean:
            extra_args += "--clean "
            pass
        if opts.no_changelog:
            extra_args += "--no-changelog "
            pass
        if opts.noinit:
            extra_args += "--noinit "
            pass
        if opts.nochecks:
            extra_args += "--nochecks "
            pass
        if opts.no_verify:
            extra_args += "--no-verify "
            pass
        if opts.keep_pkgs:
            extra_args += "--keep-pkgs=%s " % opts.keep_pkgs
            pass
        if opts.jobs:
            extra_args += "--jobs=%s " % opts.jobs
            pass
        if opts.userootforbuild:
            extra_args += "--userootforbuild "
            pass
        if opts.local_package:
            extra_args += "--local_package "
            pass
        if opts.debuginfo:
            extra_args += "--debuginfo "
            pass
        if opts.baselibs:
            extra_args += "--baselibs "
            pass
        if opts.satisfy_from_cache_liberally:
            extra_args += "--satisfy-from-cache-liberally=%s " % opts.satisfy_from_cache_liberally
            pass
    
        extra_args.strip()

        return extra_args
    
    def get_reported_arch():
        import commands
        # We may need to teach this about platforms not part of the x86 family...
        status, reported_arch = commands.getstatusoutput("uname -i")
        if status != 0:
            raise
        if reported_arch in ["i386", "i586", "i686"]:
            arch = "i586"
        else:
            arch = reported_arch
            pass
        return arch

    def get_log_output(full_chroot_path):
        osc_output_file_path = "%s/%s" % (full_chroot_path, ".build.log")
        osc_output_file = open(osc_output_file_path)
        output = osc_output_file.read()
        osc_output_file.close()
    
        return output
    
    import commands
    import os
    import os.path
    import shlex
    import sys
    import time

    import oscpluginsgnome.bicommand as bc
    import oscpluginsgnome.builtpaclist as bpl
    import oscpluginsgnome.platforms as plats

    xdate = commands.getoutput("date +%s")

    platform = platform.lower()

    # from osc submitreq
    p = findpacs(os.curdir)[0]
    src_project = p.prjname
    src_package = p.name

    plat = plats.convert(platform, src_project)

    arch = get_reported_arch()

    def check_dir(dir, var):
        if not os.path.exists(dir):
            print "Directory '%s' does not exist, but you've defined it in" % dir
            print "~/.oscrc:%s.  Please create it." % var
            print 'Try "mkdir -p %s"' % dir
            sys.exit(1)
            pass
        pass

    try:
        chroot_dir = os.path.expanduser(conf.config['eb-chroot-dir'])
        package_cache_dir = os.path.expanduser(conf.config['eb-packagecache-dir'])
        
        check_dir(chroot_dir, 'eb-chroot-dir')
        check_dir(package_cache_dir, 'eb-packagecache-dir')
    except KeyError:
        print "You need to define `eb-chroot-dir' and/or `eb-packagecache-dir' in"
        print "~/.oscrc"
        sys.exit(1)
        pass

    try:
        extra_default_packages = conf.config['eb-extra-packs']
    except KeyError:
        print "You haven't defined any extra packages to be installed.  This is"
        print "optional; however, if you wish to do so, set `eb-extra-packs' in"
        print "~/.oscrc."
        time.sleep(1)
        pass

    try:
        local_packages = os.path.expanduser(conf.config['eb-local-packs-dir'])

        check_dir(local_packages, 'eb-local-packs-dir')

        pc_flag = "-p %s/%s" % (local_packages, plat)
    except KeyError:
        print "You haven't specified a local package directory.  This is optional;"
        print "however, if you wish to do so, set `eb-local-packs-dir' in ~/.oscrc."
        pc_flag = ""
        time.sleep(1)
        pass

    # figure out the chroot
    status, me = commands.getstatusoutput("whoami")
    if status != 0:
        raise

    # My chroot naming scheme is teh suxks, but i'm sticking with it
    if platform == "factory":
        full_chroot_path = "%s/%s-%s" % (chroot_dir, me, arch)
    else:
        full_chroot_path = "%s/%s-%s-%s" % (chroot_dir, me, platform, arch)
        pass

    os.putenv("OSC_BUILD_ROOT", full_chroot_path)
    os.putenv("OSC_PACKAGE_CACHE", package_cache_dir)
    # I think OSC_PACKAGECACHEDIR was added later.  Le sigh.
    os.putenv("OSC_PACKAGECACHEDIR", package_cache_dir)

    extra_pacs = ""
    for pac in extra_default_packages.split():
        extra_pacs = extra_pacs + "-x %s " % pac
        pass
    extra_pacs = extra_pacs.strip()

    """OSC_BUILD_ROOT=/abuild/maw OSC_PACKAGE_CACHE=/cache/packagecache osc build -x ccache -x emacs-nox -p /cache/packages/factory/ openSUSE_Factory i586 *.spec --no-verify"""

    extra_args = frob_extra_args(opts)

    osc_cmd = "osc build %s %s %s %s %s" % (extra_pacs, pc_flag, plat, arch, spec)

    if extra_args != "":
        osc_cmd = "%s %s" % (osc_cmd, extra_args)
        pass
    
    if opts.verbose:
        print "we've got: '%s'" % osc_cmd
        pass

    extra_lines = ["We'll be pulling the log from %s" % "%s/%s" % (full_chroot_path, ".build.log"),
                   "This logfile will be generated by the following command: %s\n" % osc_cmd,
                   "OSC_BUILD_ROOT=%s\n" % full_chroot_path,
                   "OSC_PACKAGE_CACHE=%s\n" % package_cache_dir,
                   "OSC_PACKAGECACHEDIR=%s\n" % package_cache_dir,
                   "\n"]

    for line in extra_lines:
        print line

    pid = os.fork()

    if pid > 0:
        try:
            status = os.waitpid(pid, 0)
        except KeyboardInterrupt:
            import signal
            os.kill(pid, signal.SIGTERM)
            
            sys.exit(0)
    elif pid == 0:
        splat = shlex.split(osc_cmd)

        os.execvp(splat[0], splat)
    else:
        raise "os.fork failed"

    # FIXME: Do something with status here

    osc_output = get_log_output(full_chroot_path)

    packages = bpl.get_builtpaclist(full_chroot_path, osc_output.split('\n'))

    if opts.verbose:
        print "copying packages to working directory..."
        pass
    for pack in packages:
        newname = "1built_%s-%s" % (xdate, os.path.basename(pack))
        cp_cmd = "cp %s %s" % (pack, newname)
        status, output = commands.getstatusoutput(cp_cmd)
        pass

    logfile_name = "0logfile-mawbuild-%s.out" % xdate
    logfile = open(logfile_name, "w")
    # Should add some information on how osc mawbuild was invoked
    
    for line in extra_lines:
        logfile.write(line)
        
    for line in osc_output.split('\n'):
        logfile.write("%s\n" % line)
        pass
    logfile.close()

    pass
