from osc import cmdln

# FIXME: The cached output stuff is now duplicated in osc-diffreq.py and
# osc-submitreq.py.  It should go elsewhere.

# FIXME: There should be a way to cache more stuff, too.

@cmdln.alias("sl")
@cmdln.option('-u', '--use-cached', action='store_true',
              help="Use cached results, even if they're old.")
@cmdln.option('-s', '--state', default='new',
              help='only list requests in one of the comma separated given states [default=new]')
@cmdln.option('-r', '--reverse', action='store_true',
              help="List requests in reverse order (how `osc submitreq list` does it).")
@cmdln.option("-p", "--project", metavar="project",
              help="Override your favourite project")
@cmdln.option('-f', '--force', action='store_true',
              help="Force a download of new pending reqs.")
@cmdln.option('-e', '--everything', action='store_true',
              help="Show everything, even when the results are uninteresting.")
@cmdln.option('-d', '--dont-cache', action='store_true',
              help="Don't cache diffs.")
def do_submitreqlist(self, subcmd, opts):
    """${cmd_name}: List submissions waiting for acceptance or rejection.

    This command can default to using your favorite projects, if you set
    'favorite-projects' in ~/.oscrc.  Something along the lines of:

        favorite-projects = GNOME:Factory mozilla:Factory

    ${cmd_usage}
    ${cmd_option_list}
    """

    import commands
    import math
    import osc.core
    import os
    import oscpluginsgnome.cache
    import time

    if opts.project:
        favorite_projects = [opts.project]
    else:
        try:
            favorite_projects = conf.config['favorite-projects'].split()
        except KeyError:
            print "Set 'favorite-projects' in ~/.oscrc"
            print ""
            print "Something along the lines of:"
            print ""
            print "    favorite-projects = GNOME:Factory mozilla:Factory"
            print ""
            if conf.config['favorite-project']:
                print "You've already set favorite-project, but this has been changed,"
                print "so all you need to do is add an 's'.  You can also add more projects,"
                print "if you wish."
                pass
            return
        pass

    def prefix(p):
        return "%s;" % p

    for fp in favorite_projects:

        def sw(s):
            if s.startswith(prefix(fp)):
                return True
            return False

        def dropper(s):
            return int(s.lstrip(prefix(fp)))

        def recent(s):
            now = int(math.floor(time.time()))
            # We expire the cache after 5 minutes.
            # FIXME: This time might not be ideal, though.
            if opts.use_cached:
                return True
            if now < (s + (60 * 5)):
                return True
            return False

        command = 'submitreqlist'

        sl_cache_dir = oscpluginsgnome.cache.get_cache_dir(command)
        files = os.listdir(sl_cache_dir)
        files = filter(sw, files)
        times = map(dropper, files)
        current = filter(recent, times)

        if len(current) == 0 or opts.force:
            print "Pulling down current submit requests for %s and caching them..." % fp
            # There are no results or else we want to see the new ones anyway
            apiurl = conf.config['apiurl']

            # copied from osc/commandline.py
            state_list = opts.state.split(',')
            results = get_submit_request_list(apiurl, fp, None, state_list)

            everything = ""
            
            if len(results) > 0:

                if opts.reverse:
                    rev = True
                else:
                    rev = False
                    pass
                
                results.sort(reverse=rev)
                
                for result in results:
                    lv = result.list_view()
                    print lv
                    
                    everything += lv + '\n'
                    
                    if not opts.dont_cache:
                        sd_cache_dir = oscpluginsgnome.cache.get_cache_dir('submitreqdiff')
                        
                        if "%d" % result.reqid in os.listdir(sd_cache_dir):
                            pass
                        else:
                            cmd = "osc submitreq show -d %s" % result.reqid
                            status, output = commands.getstatusoutput(cmd)
                            
                            cache_file = "%s/%s" % (sd_cache_dir, result.reqid)
                            f = open(cache_file, "w")
                            
                            f.write(output)
                            f.flush()
                            f.close()
                            pass
                        pass
                    pass
                pass
            else:
                print "* No pending requests for %s" % fp
                pass
            now = int(math.floor(time.time()))
            sl_cache_file = "%s%d" % (prefix(fp), now)
            sl_cache_file_path = "%s/%s" % (sl_cache_dir, sl_cache_file)
            f = open(sl_cache_file_path, "w")
            f.write(everything)
            f.flush()
            f.close()
            pass
        else:
            # just print the old results
            
            # There can be more than one current cached file due to previous uses of -f/--force;
            # if so, we want to use the most recent one.
            
            times.sort()
            latest_result = times[-1]
            path = sl_cache_dir + "/" +  "%s;%d" % (fp, latest_result)
            f = open(path)
            lines = f.readlines()
            if len(lines) == 0:
                if opts.everything:
                    print "Showing cached results for %s:" % fp
                    print "* No pending requests for %s" % fp
                    pass
                pass
            else:
                print "Showing cached results for %s:" % fp
                for line in lines:
                    print line.strip('\n')
                    pass
                pass
            pass
        pass
    pass

